/**************************************************************************
*
*                       Version 2  Vendredi 8 Juin 1990 11:34:38 a Paris
*
*                  Test I/O en bibliotheque standard et
*                       appels systeme directs
*     Ce source permet de generer des executables executant des I/O
*     soit en appels bibliotheques standards UNIX avec 
*     fermeture des fichiers entre ecriture et lecture ou par appels
*     systemes directs ( SYS )
*     Dans une premiere mesure l'ecriture se fait sequentiellement mais 
*     la lecture est faite par alternat entre le debut et la fin :
*     1er bloc puis dernier puis 2nd puis avant-dernier etc ...
*     Dans une deuxieme mesure l'ecriture ( d'un nouveau fichier !!) toujours
*     sequentielle est suivie d'une lecture sequentielle simple .
*     En quelque sorte on a la fourchette des temps de lecture possibles .
*     Toutes ces operations sont faites par blocs de taille fixe ( 100 bytes
*     par defaut ) .
*     Les temps User et Systeme sont affiches separement pour l'ecriture et
*     la lecture.
*
*        Ils acceptent 1 argument : le nombre de blocs.
*
*        On peut modifier la taille des blocs ( Macro TAILLEBLOC ).
*        Pour adaptation a d'autres systeme que celui de la mise au point
*        editer et modifier HZ ou compiler avec -DHZ=XX ( XX est votre HZ ).
*
*                            Michel GAUDET 01/90
*
**************************************************************************/
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/times.h>

#ifndef TAILLEBLOC
#define TAILLEBLOC  100-sizeof(int)
#endif

#ifndef HZ
#define HZ	60	/* Ticks = 1801 (methode setrlimit) */
#endif

typedef struct {
	int numreg ;
	char info[TAILLEBLOC] ;
} RECORD ;

RECORD rec ;

#ifndef SYS
FILE *fp ;
#else
int fp ;
#endif

int n, recsize ;

struct tms use ;
long start , stop , start_user, start_sys, temps_user, temps_sys ;


long time () ;
void ecrit () ;

main ( argc, argv )
int argc ; char *argv[] ;

{
	int i, limit, top, bot ;

	if ( argc != 2 ) {
	    fprintf(stderr, " Usage : %s nombre_de_blocs \n",argv[0] );
	    exit( 1 );
	    }

#ifndef SYS
	sscanf(argv[1], "%d", &n);
#else
	n= atoi(argv[1]);
#endif

	if ( n <= 0 ) {
	   fprintf(stderr, " Usage : %s nombre_de_blocs( > 0 ) \n",argv[0] );
	   exit( 1 );
	   }
	
	recsize = sizeof(RECORD) ;
	limit = (n - 1 )/2 ;

	for ( i=0;i< TAILLEBLOC - 1 ; i++ ) rec.info[i] = 'A' ;

	rec.info[TAILLEBLOC - 1] = '\0' ;
	printf(" Taille de blocs = %d   Nombre de blocs = %d\n",TAILLEBLOC + sizeof(int),n);

	printf("Pire-cas ( debut/fin )\n") ; 

	ecrit () ;

	start = time ( NULL );
	times ( &use );
	start_user = use.tms_utime ;
	start_sys = use.tms_stime ;

#ifndef NOCLOSE

#ifndef SYS
	if ( ( fp = fopen ("schmoldu.truc","r") ) == ( FILE * ) NULL ) { 
	      fprintf(stderr," Erreur fopen \n");
	      exit (1) ;
	      }
#else
	if ( ( fp = open ("schmoldu.truc",O_RDONLY , 0666 ) ) == -1 ) {
	      fprintf(stderr," Erreur open \n");
	      exit (1) ;
	      }
#endif

#endif

 	if ( unlink("schmoldu.truc") != 0 ) {
	    fprintf(stderr," Erreur unlink\n");
	    exit (1) ;
	    } 

	for ( top = 0, bot = n-1; top <= limit ; top++, bot-- ){

#ifndef SYS
	    if ( fseek(fp,(long) top * recsize, 0 ) != 0 ) {
		  fprintf(stderr," Erreur fseek en %d\n",top);
		  exit (1) ;
	          }
	    if ( fread ( &rec,recsize, 1, fp ) != 1 || rec.numreg != top ) {
		  fprintf(stderr," Erreur fread en %d : lu = %d \n",top,rec.numreg);
		  exit (1) ;
	          }
#else
	   if ( lseek(fp,(long) top * recsize,0 ) == -1 ) {
		  fprintf(stderr," Erreur lseek en %d\n",top);
	          exit (1) ;
	          }
	   if ( read(fp,&rec,recsize ) != recsize || rec.numreg != top ) {
	          fprintf(stderr," Erreur read en %d : lu = %d \n",top,rec.numreg);
		  exit (1) ;
	          }
#endif

	   if ( bot != top ){

#ifndef SYS
	       if ( fseek(fp,(long) bot * recsize, 0 ) != 0 ) {
		  fprintf(stderr," Erreur fseek en %d\n",bot);
		  exit (1) ;
	          }
	       if ( fread ( &rec,recsize, 1, fp ) != 1 || rec.numreg != bot ) {
		  fprintf(stderr," Erreur fread en %d : lu = %d \n",bot,rec.numreg);
		  exit (1) ;
	          }
#else
	       if ( lseek(fp,(long) bot * recsize, 0 ) == -1 ) {
		  fprintf(stderr," Erreur lseek en %d\n",bot);
		  exit (1) ;
	          }
	       if ( read(fp,&rec,recsize) != recsize || rec.numreg !=  bot ) {
	          fprintf(stderr," Erreur read en %d : lu = %d \n",bot,rec.numreg);
		  exit (1) ;
	          }
#endif
	       }
	   }

#ifndef SYS
	fclose ( fp );
#else
	close ( fp );
#endif

	stop = time ( NULL );
	times( &use );
	temps_user = use.tms_utime - start_user ;
	temps_sys = use.tms_stime - start_sys ;
	printf("  Lecture  ( en sec ) user : %8.3f  ",( float ) temps_user / HZ );
	printf("  syst : %8.3f  ",( float ) temps_sys / HZ );
	printf("  real : %5ld\n",  stop - start ) ;

	printf("Meilleur-cas ( sequentiel )\n") ;

	ecrit () ;
			 
	start = time ( NULL );
	times ( &use );
	start_user = use.tms_utime ;
	start_sys = use.tms_stime ;

#ifndef NOCLOSE

#ifndef SYS
	if ( ( fp = fopen ("schmoldu.truc","r") ) == ( FILE * ) NULL ) { 
	      fprintf(stderr," Erreur fopen \n");
	      exit (1) ;
	      }
#else
	if ( ( fp = open ("schmoldu.truc",O_RDONLY , 0666 ) ) == -1 ) {
	      fprintf(stderr," Erreur open \n");
	      exit (1) ;
	      }
#endif

#endif

 	if ( unlink("schmoldu.truc") != 0 ) {
	    fprintf(stderr," Erreur unlink\n");
	    exit (1) ;
	    } 

	for ( i=0; i < n ; i++ ) {
#ifndef SYS
	    if ( fread(&rec,recsize,1,fp ) != 1 ||  rec.numreg != i ) {
		  fprintf(stderr," Erreur fwrite en %d \n",i);
		  exit (1) ;
	          }
#else
	    if ( read(fp,&rec,recsize ) != recsize  ||  rec.numreg != i ) {
		  fprintf(stderr," Erreur write en %d \n",i);
		  exit (1) ;
                  }
#endif
	    }

#ifndef SYS
	if ( fclose ( fp ) != 0 ) {
	    fprintf(stderr," Erreur fclose\n");
	    exit (1) ;
	    }
#else
	if ( close ( fp ) != 0 ) {
	    fprintf(stderr," Erreur close\n");
	    exit (1) ;
	    }
#endif

	stop = time ( NULL);
	times ( &use);
	temps_user = use.tms_utime - start_user ;
	temps_sys =  use.tms_stime - start_sys ;
	printf("  Lecture  ( en sec ) user : %8.3f  ", ( float ) temps_user / HZ );
	printf("  syst : %8.3f  ",( float ) temps_sys / HZ );
	printf("  real : %5ld\n", stop - start ) ;


}

void ecrit ()

{
int i ;
	start = time ( NULL );
	times( &use );
	start_user = use.tms_utime ;
	start_sys = use.tms_stime ;

#ifndef SYS
	if ( ( fp = fopen ("schmoldu.truc","w+") ) == ( FILE *) NULL ) { 
		fprintf(stderr," Erreur creation fopen \n");
		exit (1) ;
		}
#else
	if ( ( fp = open ("schmoldu.truc", O_CREAT | O_RDWR ,
	                   0666 ) ) == -1 ) {
		  fprintf(stderr," Erreur creation open \n");
		  exit (1) ;
		}
#endif

	for ( i=0; i < n ; i++ ) {
	    rec.numreg = i ;
#ifndef SYS
	    if ( fwrite(&rec,recsize,1,fp ) != 1 ) {
		  fprintf(stderr," Erreur fwrite en %d \n",i);
		  exit (1) ;
	          }
#else
	    if ( write(fp,&rec,recsize ) != recsize ) {
		  fprintf(stderr," Erreur write en %d \n",i);
		  exit (1) ;
                  }
#endif
	    }

#ifndef NOCLOSE

#ifndef SYS
	if ( fclose ( fp ) != 0 ) {
	    fprintf(stderr," Erreur fclose\n");
	    exit (1) ;
	    }
#else
	if ( close ( fp ) != 0 ) {
	    fprintf(stderr," Erreur close\n");
	    exit (1) ;
	    }
#endif

#endif

	stop = time ( NULL);
	times ( &use);
	temps_user = use.tms_utime - start_user ;
	temps_sys =  use.tms_stime - start_sys ;
	printf("  Ecriture ( en sec ) user : %8.3f  ", ( float ) temps_user / HZ );
	printf("  syst : %8.3f  ",( float ) temps_sys / HZ );
	printf("  real : %5ld\n", stop - start ) ;

}
