#				SSBA 2.0F
#
# Makefile : bsd.mk
#
##############################################################################
SHELL=/bin/sh

.PRECIOUS:	run
.IGNORE:	

PRINTER	=	lp
PRINTOPT=	-l66
PAGER	=	more
COMPILER=	cc
LOCALOPT=	-O
FLOPT=		
SYSTEM	=	-DVMUNIX -DBSD4v3 -DBSD4v2

LABEL	=	

CFLAGS	=	$(LOCALOPT) $(FLOPT)
CC	=	$(COMPILER) $(CFLAGS)

NAME	=	bsd
README	=	${NAME}.doc ${NAME}.files
MAKEFILE=	${NAME}.mk
CONFIG	=	${NAME}.cf
RUN	=	${NAME}.run
JOU	=	${NAME}.jou
LOG	=	${NAME}.log
RES	=	${NAME}.res
KILL	=	${NAME}.kill
TAR	=	${NAME}.tar
AWK	=	${NAME}1.awk ${NAME}2.awk ${NAME}3.awk ${NAME}4.awk
SHAR 	=	${NAME}.shar
BANNER 	=	${NAME}.ban
SSBALOG	=	../results/ssba.log
SSBASIZE=	../results/ssba.size
TRACE	=	1>>$(JOU) 2>&1
NUL	=	1>/dev/null 2>&1
SOURCES	=	bi.c contswit.c execs.c forkes.c gau.c nu.c\
		pipeback.c pipedis.c pipeself.c ran.c\
		seq.c signocsw.c syscall.c
OBJECTS	=	bi.o contswit.o execs.o forkes.o gau.o nu.o\
		pipeback.o pipedis.o pipeself.o ran.o\
		seq.o signocsw.o syscall.o
TARGETS	=	bi contswit execs forkes gau nu\
		pipeback pipedis pipeself ran\
		seq signocsw syscall
ARCHIVES=	$(TAR) $(SHAR)
VOLATILE=	$(JOU) $(LOG) $(RES) $(KILL) $(OBJECTS) $(TARGETS)\
		$(ARCHIVES) core
SCRIPTS =	$(MAKEFILE) $(CONFIG) $(RUN) $(AWK)
FILES	=	$(BANNER) $(README) $(SCRIPTS) $(SOURCES)

conf:
		@chmod u+x $(CONFIG)
		@./$(CONFIG)

compile:	$(TARGETS)

run:
		@./$(RUN)
sizes:
		@ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:
		@rm -f $(VOLATILE) $(NUL)
readme:
		@$(PAGER) $(README)
print:
		@pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:
		@pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:
		@tar cf $(TAR) $(FILES)
shar:
		@shar $(FILES) > $(SHAR)
force:

$(BANNER):
		@banner $(NAME) >$@

bi:		bi.c
	   	$(CC) $@.c -o $@		$(TRACE)
contswit:	contswit.c
	   	$(CC) $@.c -o $@		$(TRACE)
execs:		execs.c
	   	$(CC) $@.c -o $@		$(TRACE)
forkes:		forkes.c
	   	$(CC) $@.c -o $@		$(TRACE)
nu:		nu.c
	   	$(CC) $@.c -o $@		$(TRACE)
pipeback:	pipeback.c
	   	$(CC) $@.c -o $@		$(TRACE)
pipedis:	pipedis.c
	   	$(CC) $@.c -o $@		$(TRACE)
pipeself:	pipeself.c
	   	$(CC) $@.c -o $@		$(TRACE)
ran:		ran.c
	   	$(CC) $@.c -o $@		$(TRACE)
seq:		seq.c
	   	$(CC) $@.c -o $@		$(TRACE)
signocsw:	signocsw.c
	   	$(CC) $@.c -o $@		$(TRACE)
syscall:	syscall.c
	   	$(CC) $@.c -o $@		$(TRACE)
gau:		gau.c
	   	$(CC) $@.c -lm -o $@		$(TRACE)
