#                               SSBA 2.0F
#
# Makefile : iobench.mk
#
########################################################################
SHELL=/bin/sh

.PRECIOUS:      run
.IGNORE:

PRINTER   =       lp
PRINTOPT=        -l66
PAGER       =       more
COMPILER=        cc
LOCALOPT=        -O
FLOPT=              
SYSTEM=         -DVMUNIX -DBSD4v2
CFLAGS  =       $(LOCALOPT) $(FLOPT)
CC      =       $(COMPILER) $(CFLAGS)
NAME    =       iobench
README  =       ${NAME}.doc ${NAME}.files
MAKEFILE=       ${NAME}.mk
CONFIG  =       ${NAME}.cf
RUN     =       ${NAME}.run
RES     =       ${NAME}.res
LOG     =       ${NAME}.log
KILL    =       ${NAME}.kill
JOU     =       ${NAME}.jou
TAR     =       ${NAME}.tar
SHAR    =       ${NAME}.shar
BANNER  =       ${NAME}.ban
SSBASIZE=       ../results/ssba.size
SSBALOG =       ../results/ssba.log
TRACE   =       1>>$(JOU) 2>&1
NUL     =       1>/dev/null 2>&1
SOURCES =       ioexterns.c iobench.c iosubs.c iorandom.c ioversion.c unix_tod.c iolocks.c iouser.c unix_subs.c ioflush.c ioparams.c
OBJECTS =       iosubs.o unix_subs.o iolocks.o unix_tod.o ioexterns.o iobench.o ioversion.o ioparams.o ioflush.o iouser.o iorandom.o
TARGETS =       ioexterns iobench iosubs iorandom ioversion unix_tod iolocks iouser unix_subs ioflush ioparams
ARCHIVES=       $(TAR) $(SHAR)
VOLATILE=       $(JOU) $(LOG) $(RES) $(TARGETS) $(KILL) $(ARCHIVES) $(OBJECTS) core
SCRIPTS =       $(MAKEFILE) $(CONFIG) $(RUN) t-test
FILES   =       $(BANNER) $(README) $(SCRIPTS) $(SOURCES)

conf:           @chmod u+x $(CONFIG) @./$(CONFIG)
compile:        $(TARGETS)
run:            @./$(RUN) 
sizes:          @ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:          @rm -f $(VOLATILE) $(NUL)
readme:         @$(PAGER) $(README)
print:          @pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:       @pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:            @tar cf $(TAR) $(FILES)
shar:           @shar $(FILES) > $(SHAR)
force:

$(BANNER):      @banner $(NAME) >$@

iobench:     iobench.c iobench.o $(CC) $@.c iobench.o -o $@  $(TRACE)

ioexterns:   ioexterns.c ioexterns.o $(CC) $@.c ioexterns.o -o $@         $(TRACE)
iobench:     iobench.c iobench.o $(CC) $@.c iobench.o -o $@            $(TRACE)
iosubs:      iosubs.c iosubs.o $(CC) $@.c iosubs.o -o $@            $(TRACE)
iorandom:    iorandom.c iorandom.o $(CC) $@.c iorandom.o -o $@          $(TRACE)
ioversion:   ioversion.c ioversion.o $(CC) $@.c ioversion.o -o $@         $(TRACE)
unix_tod:    unix_tod.c unix_tod.o $(CC) $@.c unix_tod.o -o $@          $(TRACE)
iolocks:     iolocks.c ioclock.o $(CC) $@.c ioclock.o -o $@           $(TRACE)
iouser:      iouser.c iouser.o $(CC) $@.c iouser.o -o $@            $(TRACE) 
unix_subs:   unix_subs.c unix_subs $(CC) $@.c unix_subs.o -o $@         $(TRACE)
ioflush:     ioflush.c ioflush.o $(CC) $@.c ioflush.o -o $@           $(TRACE)
ioparams:    ioparams.c ioparams.o $(CC) $@.c ioparam.o -o $@          $(TRACE)

