#	STANFORD		SSBA 2.0F
#
# Makefile : smith.stan
#
#############################################################################
SHELL=/bin/sh

.PRECIOUS:	run
.IGNORE:	

PRINTER	=	lp
PRINTOPT=	-l66
PAGER	=	more
COMPILER=	f77
LOCALOPT=	-O
FLOPT=		

FFLAGS	=	-j
ETIME	=	../config/etime.o

FC	=	$(COMPILER) $(LOCALOPT) $(FLOPT) $(FFLAGS)
LD	=	$(COMPILER) $(LOCALOPT) $(FLOPT)

NAME	=	smith
README	=	${NAME}.doc ${NAME}.files
MAKEFILE=	${NAME}.mk ${NAME}.stan
CONFIG 	=	${NAME}.cf
RUN   	=	${NAME}.run
LOG	=	${NAME}.log
RES	=	${NAME}.res
JOU	=	${NAME}.jou
KILL	=	${NAME}.kill
TAR	=	${NAME}.tar
SHAR	=	${NAME}.shar
BANNER	=	${NAME}.ban
SSBALOG	=	../results/ssba.log
SSBASIZE=	../results/ssba.size
TRACE	=	1>>$(JOU) 2>&1
NUL	=	1>/dev/null 2>&1
SOURCES	=	${NAME}.f
OBJECTS	=	${NAME}.u
TARGETS	=	${NAME}
ARCHIVES=	$(TAR) $(SHAR)
VOLATILE=	$(JOU) $(LOG) $(RES) $(KILL) $(OBJECTS) $(TARGETS) \
		$(ARCHIVES) core
SCRIPTS	=	$(MAKEFILE) $(CONFIG) $(RUN)
FILES	=	$(BANNER) $(README) $(SCRIPTS) $(SOURCES)

conf:
		@chmod u+x $(CONFIG)
		@./$(CONFIG)

compile:	$(TARGETS)

run:
		@./$(RUN)
sizes:
		@ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:
		@rm -f $(VOLATILE) $(NUL)
readme:
		@$(PAGER) $(README)
print:
		@pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:
		@pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:
		@tar cf $(TAR) $(FILES)
shar:
		@shar $(FILES) > $(SHAR)
force:

$(BANNER):
		@banner $(NAME) >$@

${NAME}:	${NAME}.u $(ETIME)
		$(LD) $@.u $(ETIME) -o $@	$(TRACE)
		-@rm ${NAME}.u
${NAME}.u:	${NAME}.f
		$(FC) ${NAME}.f		$(TRACE)
${ETIME}:	../config/etime.c
		make $@ -f ../config/config.mk
