#				SSBA 2.0F
#
# Makefile : sprite.mk
#
##############################################################################
SHELL=/bin/sh

.PRECIOUS:	run
.IGNORE:	

PRINTER	=	lp
PRINTOPT=	-l66
PAGER	=	more
COMPILER=	cc
LOCALOPT=	-O
FLOPT=		
SYSTEM	=	-DVMUNIX -DBSD4v3 -DBSD4v2

LABEL	=	

CFLAGS	=	$(LOCALOPT) $(FLOPT)
CC	=	$(COMPILER) $(CFLAGS)

NAME	=	sprite
README	=	${NAME}.doc ${NAME}.files
MAKEFILE=	${NAME}.mk
CONFIG	=	${NAME}.cf
RUN	=	${NAME}.run
JOU	=	${NAME}.jou
LOG	=	${NAME}.log
RES	=	${NAME}.res
KILL	=	${NAME}.kill
TAR	=	${NAME}.tar
AWK	=	${NAME}.awk
SHAR 	=	${NAME}.shar
BANNER 	=	${NAME}.ban
SSBALOG	=	../results/ssba.log
SSBASIZE=	../results/ssba.size
TRACE	=	1>>$(JOU) 2>&1
NUL	=	1>/dev/null 2>&1
SOURCES	=	cwrd.c contswitch.c getpid.c pselect.c memorycopy.c spread.c\
		spritewrite.c opcl.c
OBJECTS	=	cwrd.o contswitch.o getpid.o pselect.o memorycopy.o spread.o\
		spritewrite.o opcl.o
TARGETS	=	cwrd contswitch getpid pselect memorycopy spread spritewrite\
		opcl
ARCHIVES=	$(TAR) $(SHAR)
VOLATILE=	$(JOU) $(LOG) $(RES) $(KILL) $(OBJECTS) $(TARGETS)\
		$(ARCHIVES) core
SCRIPTS =	$(MAKEFILE) $(CONFIG) $(RUN) $(AWK) spriteread
FILES	=	$(BANNER) $(README) $(SCRIPTS) $(SOURCES)

conf:
		@chmod u+x $(CONFIG)
		@./$(CONFIG)

compile:	$(TARGETS)

run:
		@./$(RUN)
sizes:
		@ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:
		@rm -f $(VOLATILE) $(NUL)
readme:
		@$(PAGER) $(README)
print:
		@pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:
		@pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:
		@tar cf $(TAR) $(FILES)
shar:
		@shar $(FILES) > $(SHAR)
force:

$(BANNER):
		@banner $(NAME) >$@

cwrd:		cwrd.c
	   	$(CC) $@.c -o $@		$(TRACE)
contswitch:	contswitch.c
	   	$(CC) $@.c -o $@		$(TRACE)
getpid:		getpid.c
	   	$(CC) $@.c -o $@		$(TRACE)
pselect:	pselect.c
	   	$(CC) $@.c -o $@		$(TRACE)
memorycopy:	memorycopy.c
	   	$(CC) $@.c -o $@		$(TRACE)
opcl:		opcl.c
	   	$(CC) $@.c -o $@		$(TRACE)
spread:		spread.c
	   	$(CC) $@.c -o $@		$(TRACE)
spritewrite:	spritewrite.c
	   	$(CC) $@.c -o $@		$(TRACE)
