/* 
 * spritewrite.c --
 *
 *	This program is a stand-alone benchmark to measure
 *	the speed of writing a file.  It should be invoked
 *	as follows:
 *
 *	spritewrite numK count
 *
 *	The program will write numK bytes to its standard
 *	output, seeking back to zero and repeating count
 *	times.  Then it will print out the write bandwidth.
 *	If omitted, count defaults to 1.
 *
 * Copyright 1989 Regents of the University of California.
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  The University of California
 * makes no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without
 * express or implied warranty.
 */

#include <stdio.h>

static char buffer[16384];

main(argc, argv)
int argc;
char **argv;
{
    int total, repeats, numK, i;

    if (argc < 2) {
	fprintf(stderr, "Usage: %s kBytes [count]\n", argv[0]);
	exit(1);
    }
    numK = atoi(argv[1]);

    if (argc == 2) {
	repeats = 1;
    } else {
	repeats = atoi(argv[2]);
    }
    total = 0;

    for ( ; repeats > 0; repeats--) {
	lseek(1, 0, 0);
	for (i = numK; i > 0; ) {
	    if (i > 16) {
		total += write(1, buffer, 16384);
		i -= 16;
	    } else {
		total += write(1, buffer, 1024*i - 1);
		i = 0;
	    }
	}
    }
}
