/*
*                            
*      Name        : scomp.c 
*                            
*      Version     : scomp.c,v 0.8 90/10/14 
*                            
*      Description : synthetic program to call gcc-syn repeatedly in a 
*                    similar fashion to the make program when it compiles
*                    gcc. This program forks processes in sequence to 
*                    execute the synthetic gcc program which in turn forks 
*                    processes to execute programs that synthsize the 
*                    programs that make up the compiler process.   
*                    
*      Written by  : Brad Nichols                                         
*                    bbn@maxine.wpi.edu                              
*
*      e-mail      : mach@cs.wpi.edu
*                   
*      Address     : Mach Research Group       
*                    Worcester Polytechnic Institute                
*                    CS Dept                                        
*                    100 Institute Road,                            
*                    Worcester MA 01609.                            
*                    U.S.A                                         
*                    (508) 831-5357                                 
*                                                                   
*      Rev History : 07/01/91
*                    Version 0.9 
*                    Jonas A. Lehmann (jlehmann@cs.wpi.edu)
*/                                                                 

#include <stdio.h>
#include <sys/types.h> 
#include <sys/wait.h>

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/* Constants                                                             */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

#define NUM_GCC_CALLS   18  /* Number of times to invoke sgcc */

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/
/* main()                                                                */
/* Start Timer -> execute sgcc -> stop timer -> repeat                   */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=*/

unsigned long	time_diff;

main(argc, argv)
     int   argc;
     char *argv[];
{
  int i,child;
  int statusp;
  char *ver;

  ver = (char *)calloc( 60, sizeof(char));
  strcpy(ver,"$Id: scomp.c,v 0.9 91/07/01 11:32:14 jle Exp Locker: jle $");
  ver[28] = 0;
  ver += 13;

	begin_timing();

  /* call sgcc as many times as defined by constant NUM_GCC_CALLS */

  for (i = 0; i < NUM_GCC_CALLS; i++) 
    {
      if ((child = fork()) == 0)  
	{
	  execvp("./sgcc",argv); 
	}
      else {
	wait(&statusp);
      }
    }

	fprintf(stderr,"Done.\n");
	end_timing();

  wpi_banner("Synthetic Compilation Benchmark", ver);
  printf("Number of gcc calls: %d\n",NUM_GCC_CALLS);
  printf("Time taken for the compilation: %d milliseconds\n", time_diff);
  printf("----------------------------------------------------\n\n\n\n");
}
