#				SSBA 2.0F
#
# Makefile : wpi.mk
#
##############################################################################
SHELL=/bin/sh

.PRECIOUS:	run
.IGNORE:	

PRINTER	=	lp
PRINTOPT=	-l66
PAGER	=	more
COMPILER=	cc
LOCALOPT=	-O
FLOPT=		
SYSTEM	=	-DVMUNIX -DBSD4v3 -DBSD4v2

LABEL	=	

CFLAGS	=	$(LOCALOPT) $(FLOPT)
CC	=	$(COMPILER) $(CFLAGS)

NAME	=	wpi
README	=	${NAME}.doc ${NAME}.files
MAKEFILE=	${NAME}.mk
CONFIG	=	${NAME}.cf
RUN	=	${NAME}.run
JOU	=	${NAME}.jou
LOG	=	${NAME}.log
RES	=	${NAME}.res
KILL	=	${NAME}.kill
TAR	=	${NAME}.tar
AWK	=	${NAME}.awk
SHAR 	=	${NAME}.shar
BANNER 	=	${NAME}.ban
SSBALOG	=	../results/ssba.log
SSBASIZE=	../results/ssba.size
TRACE	=	1>>$(JOU) 2>&1
NUL	=	1>/dev/null 2>&1
DATAS	=	random.out
HEADERS	=	timing.h jigsaw.h sdump.h
SOURCES	=	sdumpdata.c sdump.c solve.c jumble.c jigsaw.c timing.c\
		wpi_banner.c scc1.c scpp.c sgcc.c scomp.c
OBJECTS	=	sdumpdata.o sdump.o solve.o jumble.o jigsaw.o scc1.o scpp.o\
		sgcc.o scomp.o
COBJECTS=	timing.o wpi_banner.o
TARGETS	=	sdumpdata sdump jigsaw scc1 scpp sgcc scomp
ARCHIVES=	$(TAR) $(SHAR)
VOLATILE=	$(JOU) $(LOG) $(RES) $(KILL) $(OBJECTS) $(COBJECTS) $(TARGETS)\
		$(ARCHIVES) core dump.out jumble.out puzzle.out /tmp/file_*
SCRIPTS =	$(MAKEFILE) $(CONFIG) $(RUN) $(AWK)
FILES	=	$(BANNER) $(README) $(SCRIPTS) $(SOURCES) $(DATAS) $(HEADERS)

conf:
		@chmod u+x $(CONFIG)
		@./$(CONFIG)

compile:	$(TARGETS)

run:
		@./$(RUN)
sizes:
		@ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:
		@rm -f $(VOLATILE) $(NUL)
readme:
		@$(PAGER) $(README)
print:
		@pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:
		@pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:
		@tar cf $(TAR) $(FILES)
shar:
		@shar $(FILES) > $(SHAR)
force:

$(BANNER):
		@banner $(NAME) >$@

scomp:		scomp.o $(COBJECTS)
	   	$(CC) $@.o $(COBJECTS) -o $@	$(TRACE)
scomp.o:	scomp.c
	   	$(CC) -c scomp.c		$(TRACE)
jigsaw:		jigsaw.o jumble.o solve.o $(COBJECTS)
	   	$(CC) $@.o jumble.o solve.o $(COBJECTS) -o $@ -lm	$(TRACE)
jigsaw.o:	jigsaw.c jigsaw.h
	   	$(CC) -c jigsaw.c		$(TRACE)
jumble.o:	jumble.c jigsaw.h
	   	$(CC) -c jumble.c		$(TRACE)
solve.o:	solve.c jigsaw.h
	   	$(CC) -c solve.c		$(TRACE)
sdump:		sdump.o $(COBJECTS)
	   	$(CC) $@.o $(COBJECTS) -o $@	$(TRACE)
sdump.o:	sdump.c sdump.h
	   	$(CC) $(SYSTEM) -c sdump.c	$(TRACE)
wpi_banner.o:	wpi_banner.c
	   	$(CC) -c wpi_banner.c		$(TRACE)
timing.o:	timing.c timing.h
	   	$(CC) $(SYSTEM) -c timing.c	$(TRACE)
sdumpdata:	sdumpdata.c
	   	$(CC) $@.c -o $@		$(TRACE)
scc1:		scc1.c
	   	$(CC) $@.c -o $@		$(TRACE)
scpp:		scpp.c
	   	$(CC) $@.c -o $@		$(TRACE)
sgcc:		sgcc.c
	   	$(CC) $@.c -o $@		$(TRACE)
