/*********************************************************************
*
* Name        : timing.c
*
* Version     : 1.1
*
* Description : Common Standard Portable Timing Routine.
*
* Modified by : Aju John
*
* e-mail      : mach@cs.wpi.edu
*
* Address     : Mach Research Group
*               Worcester Polytechnic Institute
*               Computer Science Department
*               100 Institute Road,
*               Worcester MA 01609.
*               U.S.A
*               (508) 831-5357
* Rev History : Originally developed by Bradford Nichols,
*               late modified to make it in function call form, 
*               Sep 1990.
*                 
************************************************************************/


#include <sys/types.h>
#include <sys/stat.h>
#include "timing.h"

extern unsigned long time_diff;

begin_timing()

{


/*********************  START TIMING ***********************/
#ifdef VMUNIX
  getrusage(RUSAGE_SELF, &rusage_start);
#endif
#ifdef SysV
  times(&times_start);
#endif
#ifdef CLOCK
  clock_start = clock();
#endif
/***************************************************************/



}

end_timing()
{

/******** STOP TIMING, COMPUTE AND PRINT RESULT(S) ****************/
#ifdef VMUNIX
  getrusage(RUSAGE_SELF, &rusage_end);
  rusage_utime = (1000.0*rusage_end.ru_utime.tv_sec +
		       rusage_end.ru_utime.tv_usec/1000.0) - 
		 (1000.0*rusage_start.ru_utime.tv_sec +
	               rusage_start.ru_utime.tv_usec/1000.0);
  rusage_udiff = rusage_utime;
  rusage_stime = (1000.0*rusage_end.ru_stime.tv_sec +
                       rusage_end.ru_stime.tv_usec/1000.0)  -
                 (1000.0*rusage_start.ru_stime.tv_sec + 
                       rusage_start.ru_stime.tv_usec/1000.0);
  rusage_sdiff = rusage_stime;
  time_diff = rusage_udiff + rusage_sdiff;
#endif

#ifdef SysV
  times(&times_end);
  times_utime = 1000*(times_end.tms_utime 
		 - times_start.tms_utime)/HZ;
  times_udiff = times_utime;
  times_stime = 1000*(times_end.tms_stime 
		 - times_start.tms_stime)/HZ;
  times_sdiff = times_stime;
  time_diff = times_udiff + times_sdiff;
#endif

#ifdef CLOCK
  clock_end = clock();
  clock_diff = (clock_end - clock_start)/1000;
#endif
/********************************************************/

}
