#				SSBA 2.0F
#
# Makefile : wpidbase.mk
#
##############################################################################
SHELL=/bin/sh

.PRECIOUS:	run
.IGNORE:	

PRINTER	=	lp
PRINTOPT=	-l66
PAGER	=	more
COMPILER=	cc
LOCALOPT=	-O
FLOPT=		
SYSTEM	=	-DVMUNIX -DBSD4v3 -DBSD4v2

LABEL	=	

CFLAGS	=	$(LOCALOPT) $(FLOPT)
CC	=	$(COMPILER) $(CFLAGS)

NAME	=	wpidbase
README	=	${NAME}.doc ${NAME}.files
MAKEFILE=	${NAME}.mk
CONFIG	=	${NAME}.cf
RUN	=	${NAME}.run
JOU	=	${NAME}.jou
LOG	=	${NAME}.log
RES	=	${NAME}.res
KILL	=	${NAME}.kill
TAR	=	${NAME}.tar
SHAR 	=	${NAME}.shar
BANNER 	=	${NAME}.ban
SSBALOG	=	../results/ssba.log
SSBASIZE=	../results/ssba.size
TRACE	=	1>>$(JOU) 2>&1
NUL	=	1>/dev/null 2>&1
HEADERS	=	timing.h makedbms.h
SOURCES	=	makedbms.c server.c client.c utils.c rand_sdb.c timing.c\
		wpi_banner.c dummy_ops.c retry.c local_timing.c
OBJECTS	=	makedbms.o server.o client.o
COBJECTS=	timing.o wpi_banner.o utils.o retry.o dummy_ops.o rand_sdb.o\
		local_timing.o
TARGETS	=	makedbms server client
ARCHIVES=	$(TAR) $(SHAR)
VOLATILE=	$(JOU) $(LOG) $(RES) $(KILL) $(OBJECTS) $(COBJECTS) $(TARGETS)\
		$(ARCHIVES) core db.dat dbc_* dbs_* tmp*
SCRIPTS =	$(MAKEFILE) $(CONFIG) $(RUN)
FILES	=	$(BANNER) $(README) $(SCRIPTS) $(SOURCES) $(HEADERS)

conf:
		@chmod u+x $(CONFIG)
		@./$(CONFIG)

compile:	$(TARGETS)

run:
		@./$(RUN)
sizes:
		@ls -l $(TARGETS) | tee -a $(SSBASIZE)
clean:
		@rm -f $(VOLATILE) $(NUL)
readme:
		@$(PAGER) $(README)
print:
		@pr $(PRINTOPT) -h $(NAME) $(SCRIPTS) | $(PRINTER)
printall:
		@pr $(PRINTOPT) -h $(NAME) $(FILES) | $(PRINTER)
tar:
		@tar cf $(TAR) $(FILES)
shar:
		@shar $(FILES) > $(SHAR)
force:

$(BANNER):
		@banner $(NAME) >$@

server:		server.o $(COBJECTS)
	   	$(CC) $@.o $(COBJECTS) -o $@	$(TRACE)
client:		client.o $(COBJECTS)
	   	$(CC) $@.o $(COBJECTS) -o $@	$(TRACE)
server.o:	server.c
	   	$(CC) -c server.c		$(TRACE)
local_timing.o:	local_timing.c
	   	$(CC) $(SYSTEM) -c local_timing.c	$(TRACE)
rand_sdb.o:	rand_sdb.c
	   	$(CC) -c rand_sdb.c		$(TRACE)
dummy_ops.o:	dummy_ops.c makedbms.h
	   	$(CC) -c dummy_ops.c		$(TRACE)
retry.o:	retry.c
	   	$(CC) -c retry.c		$(TRACE)
utils.o:	utils.c
	   	$(CC) $(SYSTEM) -c utils.c	$(TRACE)
client.o:	client.c
	   	$(CC) -c client.c		$(TRACE)
wpi_banner.o:	wpi_banner.c
	   	$(CC) -c wpi_banner.c		$(TRACE)
timing.o:	timing.c timing.h
	   	$(CC) $(SYSTEM) -c timing.c		$(TRACE)
makedbms:	makedbms.c makedbms.h
	   	$(CC) $@.c -o $@		$(TRACE)
